@@ -9,15 +9,16 @@ from matplotlib.path import Path
 from matplotlib.spines import Spine
 from matplotlib.transforms import Affine2D
 from matplotlib.projections.polar import PolarAxes
-from matplotlib.patches import Circle, RegularPolygon
+from matplotlib.patches import RegularPolygon
 from matplotlib.projections import register_projection
+from mpl_toolkits.axes_grid1 import make_axes_locatable
 
 
 BAR_CMAPS = ['Blues', 'BuGn', 'gist_earth_r',
              'GnBu', 'PuBu', 'PuBuGn', 'summer_r']
 
 
-def process_axis(
+def process_axes(
         ax: plt.Axes=None,
         label:str = None,
         legend_kws:dict = None,
@@ -43,6 +44,7 @@ def process_axis(
         title_kws:dict=None,
         grid=None,
         grid_kws:dict = None,
+        tight_layout:bool = False,
 )-> plt.Axes:
     """
     processing of matplotlib Axes
@@ -50,25 +52,30 @@ def process_axis(
     Parameters
     ----------
     ax : plt.Axes
-        the axes which needs to be processed.
+        the :obj:`matplotlib.axes` axes which needs to be processed.
     label : str (default=None)
         will be used for legend
     legend_kws : dict, optional
-        dictionary of keyword arguments to ax.legend(**legend_kws)
+        dictionary of keyword arguments to :obj:`matplotlib.axes.Axes.legend`
         These include loc, fontsize, bbox_to_anchor, markerscale
     logy : bool
+        whether to convert y-axes to logrithmic scale or not
     logx : bool
+        whether to convert x-axes to logrithmic scale or not
     xlabel : str
-        label for x-axies
+        label for x-axes
     xlabel_kws : dict
-        keyword arguments for x-label ax.set_xlabel(xlabel, **xlabel_kws)
+        keyword arguments for :obj:`matplotlib.axes.Axes.set_xlabel` ax.set_xlabel(xlabel, **xlabel_kws)
     xtick_kws :
         # for axes.tick_params such as which, labelsize, colors etc
-    min_xticks :
-    max_xticks :
+    min_xticks : int
+        maximum number of ticks on x-axes
+    max_xticks : int
+        minimum number of ticks on x-axes
     ylabel : str
+        label for y-axes
     ylabel_kws : dict
-        ylabel kwargs
+        ylabel kwargs for :obj:`matplotlib.axes.Axes.set_ylabel`
     ytick_kws :
         for axes.tick_params()  such as which, labelsize, colors etc
     ylim :
@@ -77,6 +84,7 @@ def process_axis(
         whether to invert y-axes or not. It true following command will be
         executed ax.set_ylim(ax.get_ylim()[::-1])
     title : str
+        title for axes :obj:`matplotlib.axes.Axes.set_title`
     title_kws : dict
         title kwargs
     grid :
@@ -91,11 +99,13 @@ def process_axis(
         whether to show x-axes or not
     show_yaxis : bool, optional (default=True)
         whether to show y-axes or not
+    tight_layout : bool (default=False)
+        whether to execulte plt.tight_layout() or not
 
     Returns
     -------
     plt.Axes
-        the matplotlib Axes object which was passed to this function
+        the matplotlib Axes object :obj:`matplotlib.axes` which was passed to this function
     """
     if ax is None:
         ax = plt.gca()
@@ -134,13 +144,13 @@ def process_axis(
         xlabel_kws = xlabel_kws or {}
         ax.set_xlabel(xlabel, **xlabel_kws)
 
-    if top_spine:
+    if top_spine is not None:
         ax.spines['top'].set_visible(top_spine)
-    if bottom_spine:
+    if bottom_spine is not None:
         ax.spines['bottom'].set_visible(bottom_spine)
-    if right_spine:
+    if right_spine is not None:
         ax.spines['right'].set_visible(right_spine)
-    if left_spine:
+    if left_spine is not None:
         ax.spines['left'].set_visible(left_spine)
 
     if max_xticks is not None:
@@ -163,6 +173,9 @@ def process_axis(
     if not show_yaxis:
         ax.get_yaxis().set_visible(False)
 
+    if tight_layout:
+        plt.tight_layout()
+
     return ax
 
 
@@ -212,7 +225,16 @@ def kde(
         bins:int = 1000,
         cut:Union[float, Tuple[float]] = 0.5,
 )->Tuple[Union[np.ndarray, Tuple[np.ndarray, Optional[float]]], Any]:
-    """Generate Kernel Density Estimate plot using Gaussian kernels."""
+    """
+    Generate Kernel Density Estimate plot using Gaussian kernels.
+
+    parameters
+    ----------
+    y :
+    bw_method :
+    bins :
+    cut :
+    """
 
     # don't want to make whole easy_mpl dependent upon scipy
     from scipy.stats import gaussian_kde
@@ -379,9 +401,10 @@ def _rescale(y:np.ndarray, _min=0.0, _max=1.0)->np.ndarray:
 
 def version_info():
     import matplotlib
-
+    from . import __version__
     info = dict()
-    info['matplotlib'] = matplotlib._version
+    info['easy_mpl'] = __version__
+    info['matplotlib'] = matplotlib._get_version()
     info['numpy'] = np.__version__
 
     try:
@@ -390,7 +413,7 @@ def version_info():
     except Exception:
         pass
 
-    return version_info()
+    return info
 
 
 def is_dataframe(obj)->bool:
@@ -399,9 +422,16 @@ def is_dataframe(obj)->bool:
     return False
 
 
+def is_series(obj)->bool:
+    if all([hasattr(obj, attr) for attr in ["name", "index", "values"]]):
+        return True
+    return False
+
+
 def create_subplots(
-        naxes:int, ax:plt.Axes=None,
-        figsize=None,
+        naxes:int,
+        ax:plt.Axes = None,
+        figsize:tuple = None,
         **fig_kws
 )->Tuple:
 
@@ -455,3 +485,195 @@ def get_layout(naxes):
         else:
             nrows, ncols =  k, k
     return nrows, ncols
+
+
+class AddMarginalPlots(object):
+    """
+    Adds marginal plots for an axes.
+
+    parameters
+    -----------
+    x : array like
+    y : array like
+    ax : plt.Axes
+        :obj:`matplotlib.axes` on which to add the marginal plots
+    pad :
+    size :
+    hist : bool
+    hist_kws :
+    ridge_line_kws :
+    fill_kws :
+    fix_limits : bool
+
+    Examples
+    ---------
+    >>> from easy_mpl import plot
+    >>> x = np.random.normal(size=100)
+    >>> y = np.random.normal(size=100)
+    >>> e = x-y
+    >>> ax = plot(e, show=False)
+    >>> AddMarginalPlots(x, y, ax, hist=True)
+    >>> plt.show()
+    """
+    def __init__(self,
+                 x, y,
+                 ax,
+                 pad=0.25,
+                 size=0.7,
+                 hist:bool = True,
+                 hist_kws=None,
+                 ridge_line_kws=None,
+                 fill_kws=None,
+                 fix_limits:bool = True
+                 ):
+
+        self.ax = ax
+
+        if not isinstance(pad, (list, tuple)):
+            pad = [pad, pad]
+        self.pad = pad
+
+        if not isinstance(size, (list, tuple)):
+            size = [size, size]
+        self.size = size
+
+        self.hist = hist
+
+        HIST_KWS = self.verify_kws(hist_kws)
+        self.ridge_line_kws = self.verify_kws(ridge_line_kws)
+        self.fill_kws = self.verify_kws(fill_kws)
+
+        self.fix_limits = fix_limits
+
+        self.divider = make_axes_locatable(ax)
+
+        axHistx = self.add_ax_marg_x(x, hist_kws=HIST_KWS[0])
+        axHisty = self.add_ax_marg_y(y_data=y, hist_kws=HIST_KWS[1])
+
+        # make some labels invisible
+        plt.setp(axHistx.get_xticklabels() + axHisty.get_yticklabels(),
+                 visible=False)
+
+        despine_axes(ax, keep=["left", "bottom"])
+
+    def add_ax_marg_x(self,
+                      x_data,
+                      hist_kws:dict=None
+                      ):
+
+        line_kws = self._get_line_kws(self.ridge_line_kws[0])
+        fill_kws = self._get_fill_kws(self.ridge_line_kws[1])
+
+        if self.fix_limits:
+            xlim = np.array(self.ax.get_xlim()) * 1.05
+
+        _hist_kws = {"linewidth":0.5, "edgecolor":"k"}
+        if hist_kws is not None:
+            _hist_kws.update(hist_kws)
+
+        new_axes = self.divider.append_axes("top", self.size[0],
+                                       pad=self.pad[0], sharex=self.ax)
+        despine_axes(new_axes, keep="bottom")
+        new_axes.set_yticks([])
+
+        if self.hist:
+            # we draw histogram on old axes so that kde line
+            # comes on top
+            ax2 = new_axes.twinx()
+            new_axes.hist(x_data, **_hist_kws)
+            despine_axes(ax2)
+            ax2.set_yticks([])
+        else:
+            ax2 = new_axes
+
+        ind, data = kde(x_data, cut=0.2)
+        ax2.plot(ind, data, **line_kws)
+
+        if not self.hist:
+            ax2.fill_between(ind, data, **fill_kws)
+
+        if self.fix_limits:
+            self.ax.set_xlim(*xlim.tolist())
+
+        return new_axes
+
+    def add_ax_marg_y(self,
+                      y_data,
+                      hist_kws: dict = None
+                      ):
+
+        line_kws = self._get_line_kws(self.ridge_line_kws[1])
+        fill_kws = self._get_fill_kws(self.fill_kws[1])
+
+        if self.fix_limits:
+            ylim = self.ax.get_ylim()
+
+        _hist_kws = {"linewidth":0.5,
+                     "edgecolor":"k",
+                     'orientation':'horizontal'}
+        if hist_kws is not None:
+            _hist_kws.update(hist_kws)
+
+        new_axes = self.divider.append_axes("right",
+                                       self.size[1],
+                                       pad=self.pad[1],
+                                       sharey=self.ax)
+        despine_axes(new_axes, keep="left")
+        new_axes.set_xticks([])
+
+        if self.hist:
+            ax2 = new_axes.twiny()
+            new_axes.hist(y_data, **_hist_kws)
+            despine_axes(ax2)
+            ax2.set_xticks([])
+        else:
+            ax2 = new_axes
+
+        ind, data = kde(y_data, cut=0.2)
+        ax2.plot(data, ind, **line_kws)
+
+        if not self.hist:
+            ax2.fill_betweenx(ind, data, **fill_kws)
+
+        if self.fix_limits:
+            self.ax.set_ylim(ylim)
+
+        return new_axes
+
+    @staticmethod
+    def _get_line_kws(line_kws)->dict:
+        _line_kws = {'color': 'k', 'lw': 1.0}
+        if line_kws is not None:
+            _line_kws.update(line_kws)
+        return _line_kws
+
+    @staticmethod
+    def _get_fill_kws(fill_kws)->dict:
+        _fill_kws = {"alpha": 0.5, 'color':'r'}
+        if fill_kws is not None:
+            _fill_kws.update(fill_kws)
+        return _fill_kws
+
+    @staticmethod
+    def verify_kws(kws=None):
+        if kws is not None and not isinstance(kws, list):
+            assert isinstance(kws, dict)
+            kws = [kws, kws]
+
+        elif kws is None:
+            kws = [None, None]
+
+        assert len(kws) == 2
+        return kws
+
+
+def despine_axes(axes, keep=None):
+
+    if not isinstance(keep, list):
+        keep = [keep]
+
+    spines = ["top", "bottom", "right", "left"]
+    for s in spines:
+        if s not in keep:
+            axes.spines[s].set_visible(False)
+    return
